local VirtualChannel = -1
local CurrentChannel = channel.current()
local RclChannel = -1

local ChannelByDigit = {}

ChannelByDigit.X = 548 --  X = 720-72-100 
ChannelByDigit.Y =  57
ChannelByDigit.W = 100
ChannelByDigit.H =  50
ChannelByDigit.BORDER_W = 4

local channelByDigit = window:new(ChannelByDigit.X
                                , ChannelByDigit.Y
                                , ChannelByDigit.W
                                , ChannelByDigit.H
                                , Color.DarkBlue
                                , Color.Blue
                                , ChannelByDigit.BORDER_W)
local maskedLabel = label:newMasked(ChannelByDigit.X +   ChannelByDigit.BORDER_W
                                  , ChannelByDigit.Y +   ChannelByDigit.BORDER_W
                                  , ChannelByDigit.W - 2*ChannelByDigit.BORDER_W
                                  , ChannelByDigit.H - 2*ChannelByDigit.BORDER_W
                                  , ""
                                  , 28
                                  , Color.White
                                  , Alignment.vCenter_hCenter
                                  , Color.DarkBlue)

window:addChild(channelByDigit, maskedLabel)

wgt:setVisible(channelByDigit, FALSE)

function setVirtualChannel(c)
	VirtualChannel = c
end

local changeChannelDigitsCount = 0

function renderChannel()
	renderInfo(InfoMenu['visible'] == 2)
	if (Volume.mute == 1) then
		renderMute()
	end
end

function changeChannel(ix,isVirtual)
	if (ix ~= -1) then -- ix can be RclChannel, and it may be unset
		if (channels.isProtected(ix) == TRUE) then
			changeChannelCallback = function() 
				renderZappingInfo(ix)
				doChangeChannel(ix, false)
			end
			PINDlg.open(changeChannelCallback)
		else
			canvas:clear(0,0, canvas_w, canvas_h)
			renderZappingInfo( ix )
			if (not isVirtual) then
				doChangeChannel( ix );
			else
				VirtualChannel = ix
			end
		end
	else
		print('[lua] no channel')
	end
end

function doChangeChannel(ix)
	RclChannel = channel.current()
	ignoreChKeys(true)
	if AppRunning.visible then
		hideApplicationRunning()
	end
	closeCC()
	view:refresh() -- in order to draw the zapping info before changing the channel
	channel.change(ix)
	VirtualChannel = -1
end

function changeChannelVirtual()
	changeChannel( VirtualChannel, false )
	VirtualChannel = -1
end

function getCurrentChannel( isVirtual )
	if (isVirtual and VirtualChannel ~= -1) then
		return VirtualChannel
	else
		return channel.current()
	end
end

function changeFactor( isVirtual, factor )
	local ch = channels.next( getCurrentChannel(isVirtual), factor )
	print('[lua] changeFactor: ch='..ch)
    changeChannel(ch,isVirtual)
end


function channelUp( isVirtual )
	changeFactor(isVirtual,1)
end

function channelDown( isVirtual )
	changeFactor(isVirtual,-1)
end

function previousChannel()
	print('[lua] previousChannel: ch='..RclChannel)
	changeChannel(RclChannel,false)
end

function nextFav()
    local nextChannel = channels.nextFavorite( channel.current(),1)
	changeChannel( nextChannel, false );
end

local ChangeChannelByDigitVisible = false
function closeChangeChannelByDigit()
	print('[lua] closeChangeChannelByDigit()')
	if ChangeChannelByDigitVisible then
		changeChannelDigitsCount = 0
		window:close(channelByDigit)
		label:reset(maskedLabel)
		ChangeChannelByDigitVisible = false
		closeChannelByDigitNav()
		mainWindow.cancelTimer(Timers.channelChange.id)
	end
end

function changeChannelByDigits()
	local str = label:getRawText(maskedLabel)
	closeChangeChannelByDigit()
	-- don't change to the same channel
	if (channels.getInfo(channel.current()).channel ~= str) then
		local channels = channels.getAll()
		for i,ch in pairs(channels) do
			if (ch.channel == str) then
				changeChannel(ch.channelID,false)
			end
		end
	end
	checkUpdates()
	changeChannelDigitsCount = 0
end

function clearIncompleteChannel()
	closeChangeChannelByDigit()
end

function changeChannelByTimer()
	changeChannelByDigits( --[[digits]] )
end

function changeChannelPressDigit( key )
	if not ChangeChannelByDigitVisible then
		print('[lua] changeChannelPressDigit(), open')
		ChangeChannelByDigitVisible = true
--		label:setText(maskedLabel, key) -- TODO: See if we can fix it. This must be done here, manually becuse the window is not open yet and therefore it can't process the keys
		openChannelByDigitNav()
		--wgt:open(channelByDigit)
		ChannelByDigitNav.processKeys(key+1)
		--changechannel:reset(wgtChangeChanneByDigit)
	end

	wgt:open(channelByDigit)
	changeChannelDigitsCount = changeChannelDigitsCount + 1

	if (changeChannelDigitsCount == 4) then
		changeChannelByDigits( --[[changeChannelDigits]] )
	else
		mainWindow.registerTimer(Timers['channelChange']['id'], Timers['channelChange']['timeout'])
	end

end

function channelDeleted(chId)
	if RclChannel == chId then
		RclChannel = -1
	end
end

----------------    ChannelByDigitNav    ----------------
ChannelByDigitNav = {
	name = "ChannelByDigitNav"
	, ignoredKeys = {
	  [KEY_INFO]         = true
	, [KEY_EPG]          = true

	, [KEY_CURSOR_DOWN]  = true
	, [KEY_CURSOR_LEFT]  = true

	, [KEY_MUTE]         = true
	, [KEY_VOLUME_UP]    = true
	, [KEY_VOLUME_DOWN]  = true

	, [KEY_MENU]         = true
	, [KEY_AUDIO]        = true
	, [KEY_CHANNEL]      = true
	, [KEY_FAVORITES]    = true
	, [KEY_SUBTITLE]     = true
	, [KEY_ASPECT]       = true
	, [KEY_CH34]         = true
	, [KEY_VIDEO]        = true
	}
	, close = closeChangeChannelByDigit
}

function ChannelByDigitNav.processKeys(key)
	print("[lua] ChannelByDigitNav.processKeys")
	if wgt:processKey(channelByDigit, key) == FALSE then
		return pkey(ChannelByDigitNav, key)
	end
	return true
end

function ChannelByDigitNav.onChannelUp()
	closeChangeChannelByDigit()
	channelUp( false )
	return true
end

function ChannelByDigitNav.onChannelDown()
	closeChangeChannelByDigit()
	channelDown( false )
	return true
end

function ChannelByDigitNav.onExit()
	closeChangeChannelByDigit()
	return true
end

function openChannelByDigitNav()
	print('[lua] openChannelByDigitNav()')
	pushCurrentWindow(ChannelByDigitNav)
end

function closeChannelByDigitNav()
	print('[lua] closeChannelByDigitNav()')
	popCurrentWindow()
end



