TabWidth    =  95
TabHeight   =  65
TabSpace    =   5
PageX       = 182
PageY       =  67
PageW       = 456
PageH       = 345
PageMargin  =  17
PageBorder  =   4
HelpHeight  =  60

MainMenu = {
	  visible=false
	, x=safe_x+10, y=safe_y+10
	, width=canvas_w-2*(safe_x+10), height=TabHeight*5 + TabSpace*4
	, visible_columns={""}
	, title='Menú principal'
	, font_size=17
	, pages = {}
	, name = 'MainMenu'

	, index=0
	, show_from=0
	, show_to=4
	, show_count=5
	, scrollable=1
	, enabled=true
	, circular=true

	, PINDlg = {
		  dlg = pindialog:new(217,167, 385,145, Color.Black, Color.Blue, 4)
		, nav = {}
		, callback = nil
		, authenticated = false
	}
}

function openMainMenu()
	disableCC()
	MainMenu.PINDlg.authenticated = false
	pushCurrentWindow(MainMenu)
	print('[lua] openMainMenu(), lockKeys()')
	lockKeys()
	if(VolumeVisible) then
		VolumeVisible = false
	end
	tabbedwindow:selectTab(MainMenu.menu, 0)
	selectPage(MainMenu.currentPage(), true)
	wgt:open(MainMenu.newtronicLogo)
	wgt:open(MainMenu.menu)
end

function hideMainMenu()
	if (wgt:isVisible(MainMenu.menu) == TRUE) then
		print ('[lua] hideMainMenu')
		Page_PIN.labelsAreActive = false
		MainMenu.PINDlg.authenticated = false
		local page = MainMenu.currentPage()
		activatePage(page, false)
		selectPage(page, false)
		wgt:close(MainMenu.menu)
		wgt:close(MainMenu.newtronicLogo)
		popCurrentWindow()
		unlockKeys()

		enableCC()
		checkUpdates()
	end
end

function MainMenu.close()
	hideMainMenu()
end

function MainMenu.processKeys(key)
	print('[lua] MainMenu.processKeys('.. key ..')')
	local page = MainMenu.currentPage()
	if menupage:isActive(page.page)==TRUE and page.processKeys and page.processKeys(key) then
		print('[lua] MainMenu, processed by page')
		return true
	end

	local ret = pkey(MainMenu, key)
	if not ret then
		ret = processKeysFrom(parent(MainMenu), key)
	end
	return ret
end

MainMenu.ignoredKeys = {
	  [KEY_MUTE]         = true
	, [KEY_VOLUME_UP]    = true
	, [KEY_VOLUME_DOWN]  = true
	, [KEY_INFO]         = true
	, [KEY_EPG]          = true
	, [KEY_ASPECT]       = true
	, [KEY_CH34]         = true
	, [KEY_VIDEO]        = true
	, [KEY_SUBTITLE]     = true
}

function activatePage(page, state)
	if page.activate then
		page.activate(state)
	end
end

function selectPage(page, state)
	if page.select then
		MainMenu.updateHelp(page)
		page.select(state)
	end
	if state then
		wgt:setVisible(page.page, TRUE)
	else
		wgt:setVisible(page.page, FALSE)
	end
end

function enterMainMenu()
	local page = MainMenu.currentPage()
	activatePage(page, true)
	MainMenu.updateHelp(page)
end

function MainMenu.currentPage()
	 return MainMenu.pages[tabbedwindow:currentPageIndex(MainMenu.menu)]
end

function MainMenu.onRight()
	print('[lua] MainMenu.rightArrow()')
	-- ask if the page is not active before entering the main menu
	-- to prevent entering the menu twice when none of the page's widgets handled the right arrow
	if (menupage:isActive(MainMenu.currentPage().page) == FALSE) then	
		enterMainMenu()
	end
	return true
end

function MainMenu.onOk()
	print('[lua] MainMenu.ok()')
	enterMainMenu()
	return true
end

function MainMenu.onMenu()
	print('[lua] MainMenu.onMenu()')
	hideMainMenu()
	return true
end

function MainMenu.onExit()
	print('[lua] MainMenu.onExit()')
	local page = MainMenu.currentPage()
	if (menupage:isActive(page.page)==0) then
		hideMainMenu()
	else
		activatePage(page, false)
		MainMenu.updateHelp(page)
	end
	return true
end

function MainMenu.onUp()
	print('[lua] MainMenu.up()')
	selectPage(MainMenu.currentPage(), false)
	tabbedwindow:selectPrevTab(MainMenu.menu)
	selectPage(MainMenu.currentPage(), true)
	return true
end

function MainMenu.onDown()
	print('[lua] MainMenu.down()')
	selectPage(MainMenu.currentPage(), false)
	tabbedwindow:selectNextTab(MainMenu.menu)
	selectPage(MainMenu.currentPage(), true)
	return true
end


--function showButtons(selected, buttonsTable)
----
--end

require "page_channels"
require "page_usb"
require "page_config"
require "page_pin"
require "page_about"

function MainMenu.updateHelp(page)
	if wgt:isVisible(MainMenu.menu) == TRUE then
		menupage:setHelp(page.page, page.currentHelp())
	end
end

Channel_PageIx = 0
MMedia_PageIx  = 1
Config_PageIx  = 2
Clave_PageIx   = 3
About_PageIx   = 4

function MainMenu.init()
	menupage:setDefaultMargin(17)
	menupage:setDefaultBorder(4)
	menupage:setDefaultTabSpace(5)
	menupage:setDefaultTabWidth(95)
	menupage:setDefaultTabHeight(65)
	menupage:setDefaultHelpHeight(60)

	local helpHeight = 65

	MainMenu.menu = tabbedwindow:new(safe_x+10, safe_y+10, canvas_w-2*(safe_x+10), TabHeight*5 + TabSpace*4 + helpHeight)
	wgt:setVisible(MainMenu.menu, FALSE)

	MainMenu.pages[Channel_PageIx] = Page_Channels
	MainMenu.pages[MMedia_PageIx]  = Page_USB
	MainMenu.pages[Config_PageIx]  = Page_Config
	MainMenu.pages[Clave_PageIx]   = Page_PIN
	MainMenu.pages[About_PageIx]   = Page_About

	for k=0,4 do
		tabbedwindow:addPage(MainMenu.menu, MainMenu.pages[k].create())
	end

	MainMenu.newtronicLogo = image:new(503, 40, 135, 23, "imgs/Logo-Newtronic.png")
	wgt:setVisible(MainMenu.newtronicLogo, FALSE)

	wgt:fixOffsets(MainMenu.menu)
end

-- MainMenu PIN dialog, a PIN dialog with a session 
function MainMenu.PINDlg.init()
	pindialog:title( MainMenu.PINDlg.dlg, "CLAVE")
	pindialog:prompt(MainMenu.PINDlg.dlg, "Por favor, ingrese la clave")
	pindialog:icon(  MainMenu.PINDlg.dlg, "imgs/Clave.png", 70,61, "imgs/Clave_Gris.png")
	pindialog:help(  MainMenu.PINDlg.dlg, "(EXIT = cancelar)")
end

function MainMenu.PINDlg.nav.processKeys(key)
	return wgt:processKey(MainMenu.PINDlg.dlg, key)
end

function MainMenu.PINDlg.open(cbk)
	MainMenu.PINDlg.callback = cbk
	if MainMenu.PINDlg.authenticated then
		MainMenu.PINDlg.callback()
	else
		pindialog:callbacks(MainMenu.PINDlg.dlg, "onCloseMenuPINdialog", "closeDialog")
		pindialog:open(MainMenu.PINDlg.dlg, control.getPIN())
		pushCurrentWindow(MainMenu.PINDlg.nav)
		lockKeys()
	end
end

function MainMenu.PINDlg.close()
	pindialog:callbacks(MainMenu.PINDlg.dlg, "", "") -- keep it clean for future uses
	popCurrentWindow()
	unlockKeys()
end

function onCloseMenuPINdialog()
	print('[lua] onCloseMenuPINdialog(), authenticated=true')
	MainMenu.PINDlg.authenticated = true
	MainMenu.PINDlg.close()
	MainMenu.PINDlg.callback()
end

-- end MainMenu PIN dialog

MainMenu.init()
MainMenu.PINDlg.init()


