----------------------------------------------------------------------------------
-- Root
-----------------------------------------------------------------------------------
RootNav = {
	  name = 'RootNav'
	, isRoot = true
	, stack_level = 1
	
	, IgnoreChKeys   = false
	, IgnoreAllKeys  = false
}

function ignoreChKeys(b)
	RootNav.IgnoreChKeys = b
end

function ignoreAllKeys(b)
	RootNav.IgnoreAllKeys = b
end

function RootNav.processKeys(key)
	print('[lua] RootNav.processKeys('.. key ..')')
	return pkey(RootNav, key)
end

function RootNav.onMenu()
	if not ApplicationRunning then
		closeCurrentWindow()
		openMainMenu()
	end
	return true
end

function RootNav.onOk()
	if not ApplicationRunning then
		changeChannelVirtual()
	end
	return true
end

function RootNav.onVolUp()
	closeToRoot()
	volumeUp()
	return true
end

function RootNav.onVolDown()
	closeToRoot()
	volumeDown()
	return true
end

function RootNav.onDigit(digit)
	closeCurrentWindow()
	changeChannelPressDigit(digit)
	return true
end

function RootNav.onChannelDown()
	if (not RootNav.IgnoreChKeys) then
		closeCurrentWindow()
		channelDown( false )
	end
	return true
end

function RootNav.onChannelUp()
	if (not RootNav.IgnoreChKeys) then
		closeCurrentWindow()
		channelUp( false )
	end
	return true
end

function RootNav.onFav()
	if (not RootNav.IgnoreChKeys) then
		closeCurrentWindow()
		nextFav()
	end
	return true
end

function RootNav.onRecall()
	if (not RootNav.IgnoreChKeys) then
		closeCurrentWindow()
		previousChannel()
	end
	return true
end

function RootNav.onDown()
	if not ApplicationRunning then
		channelDown( true )
	end
	return true
end

function RootNav.onUp()
	if not ApplicationRunning then
		channelUp( true )
	end
	return true
end

function RootNav.onMute()
	if not ApplicationRunning then
		closeToRoot()
		toggleMute()
	end
	return true
end

function RootNav.onTVmode()
	if not ApplicationRunning then
--		closeCurrentWindow()
		toggleVideoMode()
	end
	return true
end

function RootNav.onAspect()
	if not ApplicationRunning then
		closeCurrentWindow()
		toggleAspectMode()
	end
	return true
end

function RootNav.onSub()
	if not ApplicationRunning then
		closeCurrentWindow()
		toggleCaption()
	end
	return true
end

function RootNav.onAudio()
	if not ApplicationRunning then
		closeCurrentWindow()
		channel.nextAudio()
	end
	return true
end

function RootNav.onChannel()
	if not ApplicationRunning then
		closeCurrentWindow()
		toggleModulator()
	end
	return true
end

function RootNav.onInfo()
	if not ApplicationRunning then
		closeToRoot()
		renderInfo(false)
	end
	return true
end

function RootNav.onEPG()
	if (not ApplicationRunning) and channels.haveChannels() ~= 0 then
		closeCurrentWindow()
		openEPG()
	end
	return true
end

--------------------------------------------------------------------

local WindowStack = {
	  elems = {RootNav}
	, size = 1
}

function _f(func)
	if (func) then
		func()
	end
end

function printWindowStack(prefix)
	for i=WindowStack.size,1,-1 do
		local window = WindowStack.elems[i]
		if window.name then
			print('[lua] '.. prefix ..', window='..window.name)
		else
			print('[lua] '.. prefix ..', window=?')
		end
	end
end

function pushCurrentWindow(newWindow)
	print('[lua] pushCurrentWindow '..WindowStack.size)

	WindowStack.size = WindowStack.size+1
	WindowStack.elems[WindowStack.size] = newWindow
	newWindow.stack_level = WindowStack.size

	printWindowStack('pushCurrentWindow')
end

function popCurrentWindow()
	print('[lua] popCurrentWindow '..WindowStack.size)

	WindowStack.elems[WindowStack.size].stack_level = -1
	WindowStack.elems[WindowStack.size] = nil
	WindowStack.size = WindowStack.size-1
	
	printWindowStack('popCurrentWindow')
end

function CurrentWindow()
	return WindowStack.elems[WindowStack.size]
end

function parent(window)
	if window.stack_level and window.stack_level>1 then
		return WindowStack.elems[window.stack_level-1]
	else
		return nil
	end
end

function closeCurrentWindow()
	print('[lua] closeCurrentWindow()')
	_f(CurrentWindow().close)
end

function closeToRoot()
	for i=WindowStack.size,1,-1 do
		closeCurrentWindow()
	end
end

-----------------------------------------------------------------------------------

function getProcessKeys(window)
	if window and window ~= parent(window) then
		if window.processKeys then
			return window.processKeys
		else
			return getProcessKeys(parent(window))
		end
	else
		return nil
	end
end

function processKeys(key)
	return processKeysFrom(CurrentWindow(), key)
end

function processKeysFrom(window, key)
--	print('[lua] processKeysFrom('.. key ..')')
	local pKeys = getProcessKeys(window)
	if pKeys then
		return pKeys(key)
	else
		return false
	end
end

function pkey(nav, key)
	local Actions = {
	  [KEY_INFO]         = 'onInfo'
	, [KEY_EPG]          = 'onEPG'

	, [KEY_CHANNEL_UP]   = 'onChannelUp'
	, [KEY_CHANNEL_DOWN] = 'onChannelDown'

	, [KEY_MUTE]         = 'onMute'
	, [KEY_VOLUME_UP]    = 'onVolUp'
	, [KEY_VOLUME_DOWN]  = 'onVolDown'

	, [KEY_CURSOR_DOWN]  = 'onDown'
	, [KEY_CURSOR_LEFT]  = 'onLeft'
	, [KEY_CURSOR_RIGHT] = 'onRight'
	, [KEY_CURSOR_UP]    = 'onUp'
	, [KEY_ENTER]        = 'onOk'

	, [KEY_RED]          = 'onRed'
	, [KEY_GREEN]        = 'onGreen'
	, [KEY_YELLOW]       = 'onYellow'
	, [KEY_BLUE]         = 'onBlue'

	, [KEY_NUMBER_0]     = 'onDigit'
	, [KEY_NUMBER_1]     = 'onDigit'
	, [KEY_NUMBER_2]     = 'onDigit'
	, [KEY_NUMBER_3]     = 'onDigit'
	, [KEY_NUMBER_4]     = 'onDigit'
	, [KEY_NUMBER_5]     = 'onDigit'
	, [KEY_NUMBER_6]     = 'onDigit'
	, [KEY_NUMBER_7]     = 'onDigit'
	, [KEY_NUMBER_8]     = 'onDigit'
	, [KEY_NUMBER_9]     = 'onDigit'
	, [KEY_MENU]         = 'onMenu'
	, [KEY_EXIT]         = 'onExit'
	, [KEY_AUDIO]        = 'onAudio'
	, [KEY_CHANNEL]      = 'onRecall'
	, [KEY_FAVORITES]    = 'onFav'
	, [KEY_SUBTITLE]     = 'onSub'
	, [KEY_ASPECT]       = 'onAspect'
	, [KEY_CH34]         = 'onChannel'
	, [KEY_VIDEO]        = 'onTVmode'
	}

	if Actions[key] then
		if nav.name then
			print('[lua] pkey('.. nav.name .. ' ' .. key ..'), action = '..Actions[key])
		else
			print('[lua] pkey('.. key ..'), action = '..Actions[key])
		end
		local action = nav[Actions[key]]
		if (nav.ignoredKeys and nav.ignoredKeys[key]) then
			print('[lua] pkey, ignored')
			return true
		elseif action then
			print('[lua] pkey, action()')
			return action(key-1)
		else
			print('[lua] pkey, processing parent')
			return processKeysFrom(parent(nav), key )
		end
	else
		print('[lua] pkey('.. key ..'), unrecognized')
	end
	return false
end


