local X = 276
local Y = 410
local W = 240
local H =  30
local BORDER_W = 3
local FONT_SIZE = 15

local toggleLabel = label:new(0, 0, W, H, "", FONT_SIZE, Color.White, Alignment.vCenter_hCenter)
local videoToggle = window:new(X, Y, W, H, Color.DarkBlue, Color.Blue, BORDER_W)
window:addChild(videoToggle, toggleLabel)
wgt:fixOffsets(videoToggle)
wgt:setVisible(videoToggle, FALSE)

local ToggleVisible = false

function showComboToggle(text)
	if not ToggleVisible then
		openComboToggleNav()
	end

	label:setText(toggleLabel, text)
	wgt:open(videoToggle)

	ToggleVisible = true
	
	mainWindow.registerTimer(Timers['comboToggle']['id'], Timers['comboToggle']['timeout'])
end

function clearComboToggle()
	if ToggleVisible then
		closeComboToggleNav()
		mainWindow.cancelTimer(Timers.comboToggle.id)
		wgt:close(videoToggle)
		ToggleVisible = false
	end
end

-- Video modes
function toggleVideoMode()
	content,selected = display.getVideoModes(WorkingTVOut)
	if WorkingTVOut == Page_Config.CompositeVideoIndex then
		print( "[lua] toggleVideoMode, WorkingTVOut==CompositeVideoIndex")
		content = Page_Config.CompositeVideoContents
	end
	if (ToggleVisible) then
		selected = (selected + 1) % #content
		print( "[lua] toggleVideoMode, set video modo: " .. selected )
		display.setVideoMode( WorkingTVOut, selected )
	end
	print( "[lua] toggleVideoMode, toggle VM: (" .. #content .. "," .. selected .. ")" )
	showComboToggle(content[selected+1])
end

-- Aspect modes
function toggleAspectMode()
	content,selected = display.getAspectModes()
	if (ToggleVisible) then
		selected = (selected + 1) % #content
		display.setAspectMode( selected )
	end
	print( "[lua] toggle VA: (" .. #content .. "," .. selected .. ")" )
	showComboToggle(content[selected+1])
end

-- Modulators
function toggleModulator()
	content,selected = display.getModulators()
	if (ToggleVisible) then
		selected = (selected + 1) % #content
		display.setModulator( selected )
	end
	print( "[lua] toggle M: (" .. #content .. "," .. selected .. ")" )
	showComboToggle(content[selected+1])
end

-----------------------------------------------------------------------------------
ComboToggleNav = {
	  name = "ComboToggleNav"
	, close = clearComboToggle
}

function ComboToggleNav.processKeys(key)
	print('[lua] ComboToggleNav.processKeys('.. key ..')')
	if (key == KEY_TV_MODES) then
		toggleVideoMode()
	elseif (key == KEY_ASPECT) then
		toggleAspectMode()
	elseif (key == KEY_CH34) then
		toggleModulator()
	else
		return processKeysFrom(parent(ComboToggleNav), key)
	end
	return true
end

function openComboToggleNav()
	print('[lua] openComboToggleNav()')
	pushCurrentWindow(ComboToggleNav)
end

function closeComboToggleNav()
	print('[lua] closeComboToggleNav()')
	popCurrentWindow()
end


