print( "[lua] init" )

--display.setVideoMode( 2, 4 )

--Define bool constants
FALSE = 0
TRUE  = 1

canvas_w, canvas_h = canvas.size()
safe_x, safe_y, safe_w, safe_h = canvas_w/10, canvas_h/10, canvas_w/10*8, canvas_h/10*8

---------- Key Locking ------------
local KeysLocked = false
function lockKeys()
	print("[lua] lockKeys")
	if not KeysLocked and not CurrentWindow().isRoot then
		print("[lua] locking")
		KeysLocked = true
		mainWindow.lockKeys(1)
	end
end

function unlockKeys()
	print("[lua] unlockKeys")
	if KeysLocked and CurrentWindow().isRoot then
		print("[lua] unlocking")
		KeysLocked = false
		mainWindow.lockKeys(0)
	end
end

--------------------------------------------------------------------------------------------------

require "colors"

require "chs"
require "volume"
require "nav"
require "pin"
require "mainmenu"
require "parental"
require "toggles"
require "caption"

require "info"

--------------------------------------------------------------------------------------------------

ApplicationRunning = false

AppRunning = {
	  logo = nil
	, visible=false
	, visibleTime = 7000
}

function AppRunning.init()
	local img_w,img_h = 21,21
	local image = image:new(4,4, img_w,img_h, "imgs/i.png")
	AppRunning.logo = window:new( safe_x + safe_w - 2*img_w, safe_y + safe_h - (img_h+8)
	                        , img_w+8, img_h+8
	                        , Color.DarkBlue, Color.Blue,3)

	window:addChild(AppRunning.logo, image)
	wgt:fixOffsets(AppRunning.logo)
	
	wgt:setVisible(AppRunning.logo, FALSE)
	
	view.setInteractivityIndicator(AppRunning.logo)
end

function showApplicationRunning()
--	print( "[lua] showApplicationRunning()")
	AppRunning.visible = true
	view.showInteractivity(TRUE)
	mainWindow.registerTimer(Timers.appRunningTimer.id, Timers.appRunningTimer.timeout)
end

function hideApplicationRunning()
--	print( "[lua] hideApplicationRunning()")
	AppRunning.visible = false
	view.showInteractivity(FALSE)
end

AppRunning.init()

--------------------------------------------------------------------------------------------------
-- Zapper callbacks
--------------------------------------------------------------------------------------------------

function zapperApplicationLayerEnabled( isEnabled )
	print( "[lua] Applications running:" .. isEnabled )

	ApplicationRunning = isEnabled == 1
	if (ApplicationRunning) then
		disableCC()
		showApplicationRunning()
	else
		enableCC()
		checkUpdates()
	end
end

function zapperBeginScan( networks )
	Page_Channels.beginScan( networks )
end

function zapperScanNetwork( net )
	Page_Channels.scanNetwork( net )
	view.refresh()
end

function zapperProgramFound( ch )
	Page_Channels.programFound( ch )
	view.refresh()
end

function zapperEndScan()
	Page_Channels.endScan()
	if (channels.count() > 0) then
		hideMainMenu()
		changeChannel( 0, false )
	end
	view.refresh()
end

function zapperChannelChanged( chID, isShow )
	if isShow == TRUE and EPG.visible then
		EPG.channelChanged(chID)
		view.refresh()
	end
end

function zapperShowSubtitle( str )
	showCaption( str )
end

-------------------------------------------------------------------------------------
require "dialog"

-------------------------------------------------------------------------------------
require "firmupdate"

function zapperUpdateReceived( updateID )
	updateReceived(updateID)
	view.refresh()
end

-------------------------------------------------------------------------------------
local timerValue = 5000
local changeChannelTimeout = 5000

Timers = {
	  volume           = {id=2, timeout=timerValue,           action=clearVolume}
	, channelChange    = {id=4, timeout=changeChannelTimeout, action=changeChannelByTimer}
	, infoList         = {id=5, timeout=timerValue,           action=clearInfo}
	, comboToggle      = {id=7, timeout=timerValue,           action=clearComboToggle}
	, currentTime      = {id=9, timeout=60*1000,              action=nil}
	, videoModeTimeOut = {id=10, timeout=1000,                action=videoModeTick}
	, appRunningTimer  = {id=11, timeout=AppRunning.visibleTime, action=hideApplicationRunning}
}

if (channels.haveChannels() == 0) then
	openMainMenu()
	player.showSplashScreen()
end
view.refresh()

require "epg"

function OnTimerEvent( tID )
	print( "[lua] Timer " .. tID .. " expired" )
	for k,timer in pairs(Timers) do
		if (tID == timer['id']) and timer.action then
			timer.action()
			break
		end
	end
	view.refresh()
end

function OnKeyEvent( key )
	if (not processKeys( key )) then
		if (key == KEY_EXIT) then
			mainWindow.stop()
		end
	end
	view.refresh()
end

mainWindow.run()
print( "[lua] exit" )
